// Ingmar Stapel: 20201002 did some changes.
// Homepage: https://custom-build-robots.com
#include "esp_camera.h"
#include <WiFi.h>
#include <Adafruit_PWMServoDriver.h>

// The servo controller PCA9685 librarie is loaded and the servo controller itsef
// is connected to the I2C bus.
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

// Select camera model
#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

// define the variables for the I2C pins with pin numbers
#define I2C_SDA 1
#define I2C_SCL 3

#include "SSD1306Wire.h" // legacy include: `#include "SSD1306.h"`
// Initialize the OLED display using Wire library
SSD1306Wire  display(0x3c, I2C_SDA, I2C_SCL);  //D2=SDK  D1=SCK  As per labeling on NodeMCU

// Update those settings with the login information of you wifi
const char* ssid = "YYYYYYYYYYYY";
const char* password = "XXXXXXXXX";

// Those variables are used to control the servo controller
// and to update the OLED display with some status 
// and some kind of debug information.
// I knew it is not a really nice way how I programmed that... 
extern String MyIP = "---";
extern String message_1 = "---";
extern String message_2 = "---";
extern int servo_l = 10;
extern int servo_r = 10;
extern int pwmValue1 = 0;
extern int pwmValue2 = 0;
extern String motor_l = "none";
extern String motor_l_direction = "none";
extern String motor_r = "none";
extern String motor_r_direction = "none";

// That variable is used to store the IP-address inside
// for the HTML code to link the live video stream
extern String Camerafeed ="";

void startCameraServer();

void setup() {
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  Serial.println();

  // Adafruit library servo controller setup
  Wire.begin(I2C_SDA,I2C_SCL);
  pwm.begin();
  pwm.setOscillatorFrequency(27000000);
  pwm.setPWMFreq(1600);  
  Wire.setClock(400000);
  
  // OLED display setup
  Serial.println("Initializing OLED Display");
  display.init();

  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);
  
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;
  //init with high specs to pre-allocate larger buffers
  if(psramFound()){
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }

  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    message_1 = "CAM error";
    return;
  }

  //drop down frame size for higher initial frame rate
  sensor_t * s = esp_camera_sensor_get();
  s->set_framesize(s, FRAMESIZE_CIF);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    message_2 = "Try WIFI connect...";
  }
  Serial.println("");
  Serial.println("WiFi connected");
  message_2 = "WiFi connected";

  startCameraServer();

  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");
  message_1 = "Ready to go...";
}

void loop() {
  // put your main code here, to run repeatedly:

  // Draw the IP-address on the OLED display
  draw(WiFi.localIP().toString(),message_1,message_2);
  delay(1000);
 
// The following lines of code are used to set for each motor 
// (left or right) the spinning direction
  if(motor_l == "left")
  {
    if(motor_l_direction == "forward")
    {
        pwm.setPWM(1, 0, 4096 );
        pwm.setPWM(2, 4096, 0 );
        Serial.println("left | forward");      
    }
    else if (motor_l_direction == "backward")
    {
        pwm.setPWM(1, 4096, 0 );
        pwm.setPWM(2, 0, 4096 );    
        Serial.println("left | backward");   
    }
  }

  if(motor_r == "right")
  {
    if(motor_r_direction == "forward")
    {
        pwm.setPWM(4, 0, 4096 );
        pwm.setPWM(3, 4096, 0 );   
        Serial.println("right | forward");    
    }
    else if (motor_r_direction == "backward")
    {
        pwm.setPWM(4, 4096, 0 );
        pwm.setPWM(3, 0, 4096 );  
        Serial.println("right | backward");     
    }    
  }

// With that lines of code the spinning speed of each motor is set
  if(servo_l == 0)
  {
    pwm.setPWM(0, pwmValue1, pwmValue2 );
  }

  if(servo_r == 5)
  {
    pwm.setPWM(5, pwmValue1, pwmValue2 );
  }
    
}

// Function for drawing the IP-address on the OLED display
void draw(String MyIP, String message_1, String message_2) {
  
  // Each time we call that function the display will be cleared.
  display.clear();
  // The position of the text on the display is defined here
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  // The font style and size is now defined
  display.setFont(ArialMT_Plain_16);
  // The IP address is written in the first line of the display
  display.drawString(0, 0, String(MyIP));
  
  // The variable message_1 is written in the second line of the display
  display.setFont(ArialMT_Plain_16);
  display.drawString(0, 16, String(message_1));

  // The variable message_2 is written in the third line of the display  
  display.setFont(ArialMT_Plain_16);
  display.drawString(0, 32, String(message_2));

  // Now the current IP address is written in the variable Camerafeed to 
  // be used in the HTML code...
  Camerafeed = String(MyIP);
  // Now the buffer with all the information is written on the display
  display.display();

}
